@@ -6,13 +6,7 @@ use XML::Twig;
 use Storable qw(dclone);
 use vars qw($VERSION %D);
 
-
-$VERSION = 1.01;
-
-
-#----------------------------------------
-#		NEW			-
-#----------------------------------------
+$VERSION = 1.10;
 
 sub new {
 
@@ -25,6 +19,7 @@ my ($class,$self) = shift;
 $self->{twig}     = new XML::Twig(
 	start_tag_handlers 	=>
 		{nmaprun => \&_nmaprun_start_tag_hdlr },
+		
 	twig_roots 		=> {
 		scaninfo => \&_scaninfo_tag_hdlr,
 		finished => \&_finished_tag_hdlr,
@@ -34,7 +29,10 @@ $self->{twig}     = new XML::Twig(
 		addport 	=> 1,
 		debugging	=> 1,
 		verbose		=> 1,
-		hosts		=> 1
+		hosts		=> 1,
+                taskbegin       => 1,
+                taskend         => 1,
+                taskprogress    => 1
 		}
 		);
 
@@ -43,6 +41,24 @@ return $self;
 }
 
 
+#/*****************************************************************************/
+# NMAP::PARSER OBJECT METHODS
+#/*****************************************************************************/
+
+#Safe parse and parsefile will return $@ which will contain the error
+#that occured if the parsing failed (it might be empty when no error occurred)
+sub _init {
+	my $self = shift;
+	$D{callback} = $self->{callback};
+}
+
+sub _clean {
+	my $self = shift;
+	$self->{SESSION} = dclone($D{$$}{SESSION}) if($D{$$}{SESSION});
+	$self->{HOSTS}   = dclone($D{$$}{HOSTS}  ) if($D{$$}{HOSTS}  );
+	delete $D{$$};
+	delete $D{callback};
+}
 
 sub callback {
 	my $self = shift;
@@ -58,26 +74,6 @@ sub callback {
 	return $self->{callback}{is_registered};
 }
 
-
-################################################################################
-##			PARSE METHODS					      ##
-################################################################################
-#Safe parse and parsefile will return $@ which will contain the error
-#that occured if the parsing failed (it might be empty when no error occurred)
-
-sub _init {
-	my $self = shift;
-	$D{callback} = $self->{callback};
-}
-
-sub _clean {
-	my $self = shift;
-	$self->{SESSION} = dclone($D{$$}{SESSION}) if($D{$$}{SESSION});
-	$self->{HOSTS}   = dclone($D{$$}{HOSTS}  ) if($D{$$}{HOSTS}  );
-	delete $D{$$};
-	delete $D{callback};
-}
-
 sub parse {
 	my $self = shift;
 	$self->_init();
@@ -86,7 +82,7 @@ sub parse {
 	$self->_clean();
 	$self->purge;
 	return $self;
-	}
+}
 
 sub parsefile {
 	my $self = shift;
@@ -96,7 +92,7 @@ sub parsefile {
 	$self->_clean();
 	$self->purge;
 	return $self;
-	}
+}
 
 sub parsescan {
 	my $self = shift;
@@ -105,8 +101,10 @@ sub parsescan {
 	my @ips = @_;
 	my $FH;
 
-	if($args =~ /-o(?:X|N|G)/)
-	{die "[Nmap-Parser] Cannot pass option '-oX', '-oN' or '-oG' to parsecan()";}
+	if($args =~ /-o(?:X|N|G)/){
+		die "[Nmap-Parser] Cannot pass option '-oX', '-oN' or '-oG' to parsecan()";
+	}
+	
 	my $cmd = "$nmap $args -v -v -v -oX - ".(join ' ',@ips);
 	open $FH, "$cmd |" || die "[Nmap-Parser] Could not perform nmap scan - $!";
 	$self->_init();
@@ -116,22 +114,26 @@ sub parsescan {
 	$self->purge;
 	return $self;
 	
-	}
+}
 
 
-sub purge { my $self = shift; $self->{twig}->purge; return $self;  }
+sub purge {
+	my $self = shift;
+	$self->{twig}->purge;
+	return $self;
+}
 
 sub ipv4_sort {
-my $self = shift;
-
-return (sort {
-	my @ipa = split('\.',$a);
-	my @ipb = split('\.',$b);
-		$ipa[0] <=> $ipb[0] ||
-		$ipa[1] <=> $ipb[1] ||
-		$ipa[2] <=> $ipb[2] ||
-		$ipa[3] <=> $ipb[3]
-	} @_);
+	my $self = shift;
+	
+	return (sort {
+		my @ipa = split('\.',$a);
+		my @ipb = split('\.',$b);
+			$ipa[0] <=> $ipb[0] ||
+			$ipa[1] <=> $ipb[1] ||
+			$ipa[2] <=> $ipb[2] ||
+			$ipa[3] <=> $ipb[3]
+		} @_);
 }
 
 
@@ -140,7 +142,7 @@ sub get_session {
 	my $self = shift;
 	my $obj = Nmap::Parser::Session->new($self->{SESSION});
 	return $obj;
-	}
+}
 
 #HOST STUFF
 sub get_host {
@@ -182,19 +184,19 @@ sub get_ips {
 }
 
 
-################################################################################
-##				Parsing	Tag Handlers        		      ##
-################################################################################
+#/*****************************************************************************/
+# PARSING TAG HANDLERS FOR XML::TWIG
+#/*****************************************************************************/
 
 sub _nmaprun_start_tag_hdlr {
 
 	my ($twig, $tag) = @_;
 	
-$D{$$}{SESSION}{start_time}  = $tag->{'att'}->{'start'};
-$D{$$}{SESSION}{nmap_version}= $tag->{'att'}->{'version'};
-$D{$$}{SESSION}{startstr}    = $tag->{'att'}->{'startstr'};
-$D{$$}{SESSION}{xml_version} = $tag->{'att'}->{'xmloutputversion'};
-$D{$$}{SESSION}{args}        = $tag->{'att'}->{'args'};
+$D{$$}{SESSION}{start_time}  = $tag->{att}->{start};
+$D{$$}{SESSION}{nmap_version}= $tag->{att}->{version};
+$D{$$}{SESSION}{start_str}   = $tag->{att}->{startstr};
+$D{$$}{SESSION}{xml_version} = $tag->{att}->{xmloutputversion};
+$D{$$}{SESSION}{scan_args}   = $tag->{att}->{args};
 $D{$$}{SESSION}              = Nmap::Parser::Session->new($D{$$}{SESSION});
 
 $twig->purge;	
@@ -203,9 +205,9 @@ $twig->purge;
 
 sub _scaninfo_tag_hdlr {
 	my ($twig, $tag) = @_;
-	my $type        = $tag->{'att'}->{'type'};
-        my $proto       = $tag->{'att'}->{'protocol'};
-	my $numservices = $tag->{'att'}->{'numservices'};
+	my $type        = $tag->{att}->{type};
+        my $proto       = $tag->{att}->{protocol};
+	my $numservices = $tag->{att}->{numservices};
 	
 	if(defined($type)){ #there can be more than one type in one scan
 		$D{$$}{SESSION}{type}{$type}        = $proto;
@@ -218,8 +220,8 @@ sub _scaninfo_tag_hdlr {
 
 sub _finished_tag_hdlr {
 	my ($twig, $tag) = @_;
-	$D{$$}{SESSION}{finish_time} = $tag->{'att'}->{'time'};
-	$D{$$}{SESSION}{timestr}     = $tag->{'att'}->{'timestr'};
+	$D{$$}{SESSION}{finish_time} = $tag->{att}->{time};
+	$D{$$}{SESSION}{time_str}     = $tag->{att}->{timestr};
 	$twig->purge;
 }
 
@@ -235,9 +237,9 @@ sub _host_tag_hdlr {
 	my $addr_hashref;
 	$addr_hashref = __host_addr_tag_hdlr($tag);
 	#use this as the identifier
-	$id           = $addr_hashref->{'ipv4'} ||
-			$addr_hashref->{'ipv6'} ||
-			$addr_hashref->{'mac'}; #worstcase use MAC
+	$id           = $addr_hashref->{ipv4} ||
+			$addr_hashref->{ipv6} ||
+			$addr_hashref->{mac}; #worstcase use MAC
 	
 	$D{$$}{HOSTS}{$id}{addrs} = $addr_hashref;
 	
@@ -258,6 +260,9 @@ sub _host_tag_hdlr {
 	    $D{$$}{HOSTS}{$id}{tcpsequence}   = __host_tcpsequence_tag_hdlr($tag);
 	    $D{$$}{HOSTS}{$id}{ipidsequence}  = __host_ipidsequence_tag_hdlr($tag);
 	    $D{$$}{HOSTS}{$id}{tcptssequence} = __host_tcptssequence_tag_hdlr($tag);
+	    $D{$$}{HOSTS}{$id}{distance}      = __host_distance_tag_hdlr($tag); #returns simple value
+	    
+	    
 	
 	}	
 	#CREATE HOST OBJECT FOR USER
@@ -357,7 +362,7 @@ sub __host_port_tag_hdlr {
 			if(defined($proto) && defined($portid));
 		
 		#GET OWNER INFORMATION
-		$port_hashref->{$proto}{$portid}{service}{service_owner} = $owner->{att}->{name}
+		$port_hashref->{$proto}{$portid}{service}{owner} = $owner->{att}->{name}
 			if(defined($owner));
 			
 		#These are added at the end, otherwise __host_service_tag_hdlr will overwrite
@@ -381,18 +386,18 @@ sub __host_service_tag_hdlr {
 	my $portid = shift; #need a way to remember what port this service runs on
 	my $service = $tag->first_child('service[@name]');
 	my $service_hashref;
-	$service_hashref->{service_port}       = $portid;
+	$service_hashref->{port}       = $portid;
 	
 	if(defined $service){
-	$service_hashref->{service_name}       = $service->{att}->{name}  || 'unknown';
-	$service_hashref->{service_version}    = $service->{att}->{version};
-	$service_hashref->{service_product}    = $service->{att}->{product};
-	$service_hashref->{service_extrainfo}  = $service->{att}->{extrainfo};
-	$service_hashref->{service_proto}      = $service->{att}->{proto} || 'unknown';
-	$service_hashref->{service_rpcnum}     = $service->{att}->{rpcnum};
-	$service_hashref->{service_tunnel}     = $service->{att}->{tunnel};
-	$service_hashref->{service_method}     = $service->{att}->{method};
-	$service_hashref->{service_confidence} = $service->{att}->{conf};
+	$service_hashref->{name}       = $service->{att}->{name}  || 'unknown';
+	$service_hashref->{version}    = $service->{att}->{version};
+	$service_hashref->{product}    = $service->{att}->{product};
+	$service_hashref->{extrainfo}  = $service->{att}->{extrainfo};
+	$service_hashref->{proto}      = $service->{att}->{proto} || 'unknown';
+	$service_hashref->{rpcnum}     = $service->{att}->{rpcnum};
+	$service_hashref->{tunnel}     = $service->{att}->{tunnel};
+	$service_hashref->{method}     = $service->{att}->{method};
+	$service_hashref->{confidence} = $service->{att}->{conf};
 	}
 
 	return $service_hashref;
@@ -404,6 +409,7 @@ sub __host_os_tag_hdlr {
 	my $os_tag = $tag->first_child('os');
 	my $os_hashref;
 	my $portused_tag;
+	my $os_fingerprint;
 	
 	if(defined $os_tag){
 	#get the open port used to match os
@@ -414,11 +420,15 @@ sub __host_os_tag_hdlr {
 	$portused_tag = $os_tag->first_child("portused[\@state='closed']");
 	$os_hashref->{portused}{closed} = $portused_tag->{att}->{portid} if(defined $portused_tag);
 	
+	#os fingerprint
+        $os_fingerprint = $os_tag->first_child("osfingerprint");
+        $os_hashref->{os_fingerprint} = $os_fingerprint->{'att'}->{'fingerprint'} if(defined $os_fingerprint);
+
 	#This will go in Nmap::Parser::Host::OS
 	my $osmatch_index = 0;
 	for my $osmatch ($os_tag->children('osmatch')){
-		$os_hashref->{osmatch_names}   [$osmatch_index] = $osmatch->{att}->{name};
-		$os_hashref->{osmatch_accuracy}[$osmatch_index] = $osmatch->{att}->{accuracy};
+		$os_hashref->{osmatch_name}   [$osmatch_index] = $osmatch->{att}->{name};
+		$os_hashref->{osmatch_name_accuracy}[$osmatch_index] = $osmatch->{att}->{accuracy};
 		$osmatch_index++;
 		}
 	$os_hashref->{'osmatch_count'} = $osmatch_index;
@@ -430,12 +440,13 @@ sub __host_os_tag_hdlr {
 		$os_hashref->{osclass_osgen}   [$osclass_index] = $osclass->{att}->{osgen};
 		$os_hashref->{osclass_vendor}  [$osclass_index] = $osclass->{att}->{vendor};
 		$os_hashref->{osclass_type}    [$osclass_index] = $osclass->{att}->{type};
-		$os_hashref->{osclass_accuracy}[$osclass_index] = $osclass->{att}->{accuracy};
+		$os_hashref->{osclass_class_accuracy}[$osclass_index] = $osclass->{att}->{accuracy};
 		$osclass_index++;		
 	}
 	$os_hashref->{'osclass_count'} = $osclass_index;
 	}
 	
+
 	return $os_hashref;	
 	
 }
@@ -492,44 +503,67 @@ sub __host_tcptssequence_tag_hdlr {
 	return $sequence_hashref;
 	}
 
+sub __host_distance_tag_hdlr {
+	my $tag = shift;
+	my $distance = $tag->first_child('distance');
+	return undef unless($distance);
+	return $distance->{att}->{value};
+}
 
 
-################################################################################
-##			Nmap::Parser::Session			              ##
-################################################################################
+
+#/*****************************************************************************/
+# NMAP::PARSER::SESSION
+#/*****************************************************************************/
 
 package Nmap::Parser::Session;
+use vars qw($AUTOLOAD);
 
 sub new {
-my $class = shift;
-$class    = ref($class) || $class;
-my $self  =  shift      || {};
-bless ($self,$class);
-return $self;
+	my $class = shift;
+	$class    = ref($class) || $class;
+	my $self  =  shift      || {};
+	bless ($self,$class);
+	return $self;
 }
 
+#Support for:
+#start_time, start_str, finish_time, time_str, nmap_version, xml_version, scan_args
+sub AUTOLOAD {
+	(my $param = $AUTOLOAD) =~ s{.*::}{}xms;
+	return if($param eq 'DESTROY');
+	no strict 'refs';
+	*$AUTOLOAD = sub {return $_[0]->{lc $param}};	
+	goto &$AUTOLOAD;
+}
+
+
 sub numservices {
 	my $self = shift;
 	my $type = shift || ''; #(syn|ack|bounce|connect|null|xmas|window|maimon|fin|udp|ipproto)
 
-return unless(ref($self->{numservices}) eq 'HASH');
+	return unless(ref($self->{numservices}) eq 'HASH');
+
+	if($type ne ''){return $self->{numservices}{$type};}
+	else {
+		my $total = 0;
+		for (values %{$self->{numservices}}){$total +=$_;}
+		return $total;
+	}#(else) total number of services together
+}
 
-if($type ne ''){return $self->{numservices}{$type};}
-else {my $total = 0;for (values %{$self->{numservices}}){$total +=$_;}
-return $total;}#total number of services together
-	}
 
-sub start_time 		{return $_[0]->{start_time};}
-sub start_str 		{return $_[0]->{startstr};}
-sub finish_time 	{return $_[0]->{finish_time};}
-sub time_str 		{return $_[0]->{timestr};}
-sub nmap_version 	{return $_[0]->{nmap_version};}
-sub xml_version 	{return $_[0]->{xml_version};}
-sub scan_args 		{return $_[0]->{args};}
 sub scan_types 		{return sort {$a cmp $b} (keys %{$_[0]->{type}}) if(ref($_[0]->{type}) eq 'HASH');}
 sub scan_type_proto 	{return $_[1] ? $_[0]->{type}{$_[1]} : undef;}
 
+
+
+#/*****************************************************************************/
+# NMAP::PARSER::HOST
+#/*****************************************************************************/
+
 package Nmap::Parser::Host;
+use vars qw($AUTOLOAD);
 
 sub new {
 my $class = shift;
@@ -566,7 +600,7 @@ sub hostname  {
 sub all_hostnames {return @{$_[0]->{hostnames}};}
 sub extraports_state {return $_[0]->{ports}{extraports}{state};}
 sub extraports_count {return $_[0]->{ports}{extraports}{count};}
-
+sub distance {return $_[0]->{distance}; }
 
 
 sub _get_ports {
@@ -575,6 +609,10 @@ sub _get_ports {
 	my $state = lc(shift);    #open, filtered, closed or any combination
 	my @matched_ports = ();
 	
+	#if $state eq '', then tcp_ports or udp_ports was called for all ports
+	#therefore, only return the keys of all ports found
+	if($state eq ''){return sort {$a <=> $b} (keys %{ $self->{ports}{$proto} }) }
+	
 	#the port parameter can be set to either any of these also 'open|filtered'
 	#can count as 'open' and 'filetered'. Therefore I need to use a regex from now on
 	#if $param is empty, then all ports match.
@@ -606,18 +644,6 @@ sub _get_port_state {
 sub tcp_ports { return _get_ports(@_,'tcp');}
 sub udp_ports { return _get_ports(@_,'udp');}
 
-
-#not if port_state 'open|filtered' && 'open'
-sub tcp_open_ports {return _get_ports($_[0], 'open','tcp');}
-sub udp_open_ports {return _get_ports($_[0], 'open','udp');}
-
-sub tcp_filtered_ports {return _get_ports($_[0], 'filtered','tcp');}
-sub udp_filtered_ports {return _get_ports($_[0], 'filtered','udp');}
-
-sub tcp_closed_ports {return _get_ports($_[0], 'closed','tcp');}
-sub udp_closed_ports {return _get_ports($_[0], 'closed','udp');}
-
-
 sub tcp_port_count {return $_[0]->{ports}{tcp_port_count};}
 sub udp_port_count {return $_[0]->{ports}{udp_port_count};}
 
@@ -652,23 +678,40 @@ sub udp_service {
 
 sub os_sig {return Nmap::Parser::Host::OS->new($_[0]->{os});}
 
-sub tcpsequence_class  {return $_[0]->{tcpsequence}{class};}
-sub tcpsequence_values {return $_[0]->{tcpsequence}{values};}
-sub tcpsequence_index  {return $_[0]->{tcpsequence}{index};}
+
+#Support for:
+#tcpsequence_class, tcpsequence_values, tcpsequence_index,
+#ipidsequence_class, ipidsequence_values, tcptssequence_values,
+#tcptssequence_class, uptime_seconds, uptime_lastboot
+#tcp_open_ports, udp_open_ports, tcp_filtered_ports, udp_filtered_ports,
+#tcp_closed_ports, udp_closed_ports
+sub AUTOLOAD {
+	(my $param = $AUTOLOAD) =~ s{.*::}{}xms;
+	return if($param eq 'DESTROY');
+	my($type,$val) = split /_/, lc($param);
+	#splits the given method name by '_'. This will determine the function and param
+	no strict 'refs';
+	
+	if( ($type eq 'tcp' || $type eq 'udp') && ($val eq 'open' || $val eq 'filtered' || $val eq 'closed') ){
+					
+		#they must be looking for port info: tcp or udp. The $val is either open|filtered|closed
+		*$AUTOLOAD = sub {return _get_ports($_[0], $val, $type);};	
+		goto &$AUTOLOAD;
+		
+	} elsif(defined $type && defined $val) {
+		#must be one of the 'sequence' functions asking for class/values/index
+		*$AUTOLOAD = sub {return $_[0]->{$type}{$val}};	
+		goto &$AUTOLOAD;
+	} else {die '[Nmap-Parser] method ->'.$param."() not defined!\n";}
+}
 
 
-sub ipidsequence_class  {return $_[0]->{ipidsequence}{class};}
-sub ipidsequence_values {return $_[0]->{ipidsequence}{values};}
-
-
-sub tcptssequence_class  {return $_[0]->{tcptssequence}{class};}
-sub tcptssequence_values {return $_[0]->{tcptssequence}{values};}
-
-sub uptime_seconds  {return $_[0]->{uptime}{seconds};}
-sub uptime_lastboot {return $_[0]->{uptime}{lastboot};}
-
+#/*****************************************************************************/
+# NMAP::PARSER::HOST::SERVICE
+#/*****************************************************************************/
 
 package Nmap::Parser::Host::Service;
+use vars qw($AUTOLOAD);
 
 sub new {
 my $class = shift;
@@ -678,20 +721,29 @@ bless ($self,$class);
 return $self;
 }
 
-sub name 	{return $_[0]->{service_name};}
-sub port 	{return $_[0]->{service_port};}
-sub proto 	{return $_[0]->{service_proto};}
-sub rpcnum 	{return $_[0]->{service_rpcnum};}
-sub owner 	{return $_[0]->{service_owner};}
-sub version 	{return $_[0]->{service_version};}
-sub product 	{return $_[0]->{service_product};}
-sub extrainfo 	{return $_[0]->{service_extrainfo};}
-sub tunnel 	{return $_[0]->{service_tunnel};}
-sub method	{return $_[0]->{service_method};}
-sub confidence 	{return $_[0]->{service_confidence};}
+
+#Support for:
+#name port proto rpcnum owner version product extrainfo tunnel method confidence
+#this will now only load functions that will be used. This saves
+#on delay (increase speed) and memory
+
+sub AUTOLOAD {
+	(my $param = $AUTOLOAD) =~ s{.*::}{}xms;
+	return if($param eq 'DESTROY');
+	no strict 'refs';
+	
+	*$AUTOLOAD = sub {return $_[0]->{lc $param}};	
+	goto &$AUTOLOAD;
+}
+
+
+#/*****************************************************************************/
+# NMAP::PARSER::HOST::OS
+#/*****************************************************************************/
 
 
 package Nmap::Parser::Host::OS;
+use vars qw($AUTOLOAD);
 
 sub new {
 my $class = shift;
@@ -703,86 +755,47 @@ return $self;
 
 sub portused_open   {return $_[0]->{portused}{open};} 
 sub portused_closed {return $_[0]->{portused}{closed};}
+sub os_fingerprint  {return $_[0]->{os_fingerprint};}
 
 sub name_count {return $_[0]->{osmatch_count};}
-sub name_accuracy {
-	
-	my $self = shift;
-	my $index = shift || 0;
-	if($index >= $self->{osmatch_count}){
-		$index = $self->{osmatch_count}-1;
-	}
-	
-	return $self->{osmatch_accuracy}[$index];	
-}
-sub name {
-	my $self = shift;
-	my $index = shift || 0;
-	if($index >= $self->{osmatch_count}){
-		$index = $self->{osmatch_count}-1;
-	}
-	
-	return $self->{osmatch_names}[$index];
-}
 
 sub all_names {
     my $self = shift;
     @_=();
     if($self->{osclass_count} < 1){return @_;}
-    if(ref($self->{osmatch_names}) eq 'ARRAY'){
-    return sort @{$self->{osmatch_names}};}
+    if(ref($self->{osmatch_name}) eq 'ARRAY'){
+    return sort @{$self->{osmatch_name}};}
 
 } #given by decreasing accuracy
 
 sub class_count {return $_[0]->{osclass_count};}
 
-sub osfamily {
-	my $self = shift;
-	my $index = shift || 0;
-	if($index >= $self->{osclass_count}){
-		$index = $self->{osclass_count}-1;
-	}
-	
-	return $self->{osclass_osfamily}[$index];
+#Support for:
+#name,names, name_accuracy, osfamily, vendor, type, osgen, class_accuracy
+sub AUTOLOAD {
+	(my $param = $AUTOLOAD) =~ s{.*::}{}xms;
+	return if($param eq 'DESTROY');
+	no strict 'refs';
+	$param = lc($param);
 	
+	$param = 'name' if($param eq 'names');
+	if($param eq 'name' || $param eq 'name_accuracy'){
+		
+		*$AUTOLOAD = sub {_get_info($_[0],$_[1],$param,'osmatch');};		
+		goto &$AUTOLOAD;
+	} else {
+
+		*$AUTOLOAD = sub {_get_info($_[0],$_[1],$param,'osclass');};	
+		goto &$AUTOLOAD;
+	}
 }
 
-
-sub vendor {
-	my $self = shift;
-	my $index = shift || 0;
-	if($index >= $self->{osclass_count}){
-		$index = $self->{osclass_count}-1;
-	}
-	return $self->{osclass_vendor}[$index];
-	
-}
-
-sub osgen {
-	my $self = shift;
-	my $index = shift || 0;
-	if($index >= $self->{osclass_count}){
-		$index = $self->{osclass_count}-1;
-	}
-	return $self->{osclass_osgen}[$index];		
-}
-
-sub type {
-	my $self = shift;
-	my $index = shift || 0;
-	if($index >= $self->{osclass_count}){
-		$index = $self->{osclass_count}-1;
-	}
-	return $self->{osclass_type}[$index];		
-}
-
-sub class_accuracy {
-	my $self = shift;
-	my $index = shift || 0;
-	if($index >= $self->{osclass_count}){
-		$index = $self->{osclass_count}-1;
-	}
-	return $self->{osclass_accuracy}[$index];		
+sub _get_info {
+	my($self,$index,$param,$type) = @_;
+	$index ||= 0;
+	#type is either osclass or osmatch
+	if($index >= $self->{$type.'_count'}){$index = $self->{$type.'_count'}-1;}
+	return $self->{$type.'_'.$param}[$index];		
 }
 
 1;
@@ -804,7 +817,7 @@ Nmap::Parser - parse nmap scan data with perl
     #or
   $np->parsefile($file_xml);
   
-  my $session    = $np->session();
+  my $session    = $np->get_session();
     #a Nmap::Parser::Session object
     
   my $host       = $np->get_host($ip_addr);
@@ -864,6 +877,7 @@ operating system signature information (OS guessed names, classes, osfamily..etc
      |  |
      |  |-Nmap::Parser::Host::OS      -- Operating system signature information
 
+
 =head1 METHODS
 
 =head2 Nmap::Parser
@@ -1049,6 +1063,7 @@ Returns the number of extraports found.
 Returns the state of all the extraports found.
 
 =item B<hostname()>
+
 =item B<hostname($index)>
 
 As a basic call, hostname() returns the first hostname obtained for the given
@@ -1077,6 +1092,10 @@ Explicitly return the MAC address.
 
 Return the vendor information of the MAC.
 
+=item B<distance()>
+
+Return the distance (in hops) of the target machine from the machine that performed the scan.
+
 =item B<os_sig()>
 
 Returns an Nmap::Parser::Host::OS object that can be used to obtain all the
@@ -1125,11 +1144,11 @@ when the scan was performed.
 Returns the sorted list of TCP|UDP ports respectively that were scanned on this host. Optionally
 a string argument can be given to these functions to filter the list.
 
- $host->tcp_ports('open') #returns all only 'open' ports
+ $host->tcp_ports('open') #returns all only 'open' ports (even 'open|filtered')
  $host->udp_ports('open|filtered'); #matches exactly ports with 'open|filtered'
  
-Note that if a port state is set to 'open|filtered' (or any combination), it will
-be counted as an 'open' port as well as a 'filtered' one.
+I<Note that if a port state is set to 'open|filtered' (or any combination), it will
+be counted as an 'open' port as well as a 'filtered' one.>
 
 =item B<tcp_port_count()>
 
@@ -1149,21 +1168,21 @@ Returns the state of the given port, provided by the port number in $portid.
 =item B<udp_open_ports()>
 
 Returns the list of open TCP|UDP ports respectively. Note that if a port state is
-'open|filtered', it will appear on this list as well. 
+for example, 'open|filtered', it will appear on this list as well. 
 
 =item B<tcp_filtered_ports()>
 
 =item B<udp_filtered_ports()>
 
 Returns the list of filtered TCP|UDP ports respectively. Note that if a port state is
-'open|filtered', it will appear on this list as well. 
+for example, 'open|filtered', it will appear on this list as well. 
 
 =item B<tcp_closed_ports()>
 
 =item B<udp_closed_ports()>
 
 Returns the list of closed TCP|UDP ports respectively. Note that if a port state is
-'closed|filtered', it will appear on this list as well. 
+for example, 'closed|filtered', it will appear on this list as well. 
 
 =item B<tcp_service($portid)>
 
@@ -1268,6 +1287,10 @@ Returns the total number of OS class records obtained from the nmap scan.
 
 =item B<name($index)>
 
+=item B<names()>
+
+=item B<names($index)>
+
 A basic call to name() returns the OS name of the first record which is the name
 with the highest accuracy. If C<$index> is given, it returns the name for the given record. The
 index starts at 0.
@@ -1292,6 +1315,7 @@ If C<$index> is given, it returns the OS family information for the given record
 index starts at 0.
 
 =item B<osgen()>
+
 =item B<osgen($index)>
 
 A basic call to osgen() returns the OS generation information of the first record.
@@ -1308,6 +1332,10 @@ be available for all hosts.
 Returns the open port number used to help identify the OS signatures. This might
 not be available for all hosts.
 
+=item B<os_fingerprint()>
+
+Returns the OS fingerprint used to help identify the OS signatures. This might not be available for all hosts.
+
 =item B<type()>
 
 =item B<type($index)>
@@ -1330,7 +1358,8 @@ index starts at 0.
 
 I think some of us best learn from examples. These are a couple of examples to help
 create custom security audit tools using some of the nice features
-of the Nmap::Parser module. Hopefully this can double as a tutorial.
+of the Nmap::Parser module. Hopefully this can double as a tutorial. 
+More tutorials (articles) can be found at L<nmapparser.wordpress.com>
 
 =head2 Real-Time Scanning - (no better C<time()> like C<'now'>)
 
@@ -1439,16 +1468,11 @@ B<Please make sure that you submit the xml-output file of the scan which you are
 trouble.> This can be done by running your scan with the I<-oX filename.xml> nmap switch.
 Please remove any important IP addresses for security reasons.
 
-=head2 Feature Requests
-
-Please submit any requests to:
-L<http://sourceforge.net/tracker/?atid=618348&group_id=97509&func=browse>
-
 =head1 SEE ALSO
 
  nmap, XML::Twig
 
-The Nmap::Parser page can be found at: L<http://www.nmapparser.com>.
+The Nmap::Parser page can be found at: L<http://nmapparser.wordpress.com>.
 It contains the latest developments on the module. The nmap security scanner
 homepage can be found at: L<http://www.insecure.org/nmap/>.
 
@@ -1458,15 +1482,26 @@ Anthony G Persaud <apersaud@gmail.com> L<http://www.anthonypersaud.com>
 
 =head1 COPYRIGHT
 
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
+Copyright (c) <2007> <Anthony G. Persaud>
 
-This program is distributed in the hope that it will be useful, but WITHOUT ANY
-WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+MIT License
 
-L<http://www.opensource.org/licenses/gpl-license.php>
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
 
 =cut
